@@ -3,16 +3,14 @@
 #
 # File: IPTables::Parse.pm
 #
-# Purpose: Perl interface to parse iptables rulesets.
+# Purpose: Perl interface to parse iptables and ip6tables rulesets.
 #
 # Author: Michael Rash (mbr@cipherdyne.org)
 #
-# Version: 0.7
+# Version: 1.1
 #
 ##################################################################
 #
-# $Id: Parse.pm 2135 2008-01-18 03:20:40Z mbr $
-#
 
 package IPTables::Parse;
 
@@ -23,14 +21,14 @@ use strict;
 use warnings;
 use vars qw($VERSION);
 
-$VERSION = '0.7';
+$VERSION = '1.1';
 
 sub new() {
     my $class = shift;
     my %args  = @_;
 
     my $self = {
-        _iptables => $args{'iptables'} || '/sbin/iptables',
+        _iptables => $args{'iptables'} || $args{'ip6tables'} || '/sbin/iptables',
         _iptout    => $args{'iptout'}    || '/tmp/ipt.out',
         _ipterr    => $args{'ipterr'}    || '/tmp/ipt.err',
         _ipt_alarm => $args{'ipt_alarm'} || 30,
@@ -44,6 +42,10 @@ sub new() {
         unless -e $self->{'_iptables'};
     croak "[*] $self->{'_iptables'} not executable.\n"
         unless -x $self->{'_iptables'};
+
+    $self->{'_ipt_bin_name'} = 'iptables';
+    $self->{'_ipt_bin_name'} = $1 if $self->{'_iptables'} =~ m|.*/(\S+)|;
+
     bless $self, $class;
 }
 
@@ -93,6 +95,8 @@ sub chain_rules() {
 
     my $found_chain  = 0;
     my @ipt_lines = ();
+
+    ### only used for IPv4 + NAT
     my $ip_re = qr|(?:[0-2]?\d{1,2}\.){3}[0-2]?\d{1,2}|;
 
     ### array of hash refs
@@ -156,17 +160,32 @@ sub chain_rules() {
             'to_port'  => '',
             'extended' => '',
             'state'    => '',
+            'ctstate'  => '',
             'raw'      => $line
         );
 
         if ($ipt_verbose) {
+
+            ### iptables:
             ### 0     0 ACCEPT  tcp  --  eth1 * 192.168.10.3  0.0.0.0/0  tcp dpt:80
             ### 0     0 ACCEPT  tcp  --  eth1 * 192.168.10.15 0.0.0.0/0  tcp dpt:22
             ### 33 2348 ACCEPT  tcp  --  eth1 * 192.168.10.2  0.0.0.0/0  tcp dpt:22
             ### 0     0 ACCEPT  tcp  --  eth1 * 192.168.10.2  0.0.0.0/0  tcp dpt:80
             ### 0     0 DNAT    tcp  --  *    * 123.123.123.123 0.0.0.0/0 tcp dpt:55000 to:192.168.12.12:80
-            if ($line =~ m|^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+\-\-\s+
-                                (\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*(.*)|x) {
+
+            ### ip6tables:
+            ### 0     0 ACCEPT  tcp   *   *   ::/0     fe80::aa:0:1/128    tcp dpt:12345
+            ### 0     0 LOG     all   *   *   ::/0     ::/0                LOG flags 0 level 4
+
+            my $match_re = qr/^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+\-\-\s+
+                                (\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*(.*)/x;
+
+            if ($self->{'_ipt_bin_name'} eq 'ip6tables') {
+                $match_re = qr/^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+
+                                (\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*(.*)/x;
+            }
+
+            if ($line =~ $match_re) {
                 $rule{'packets'}  = $1;
                 $rule{'bytes'}    = $2;
                 $rule{'target'}   = $3;
@@ -201,41 +220,17 @@ sub chain_rules() {
                             $rule{'to_ip'}   = $1;
                             $rule{'to_port'} = $2;
                         }
-
-                        for my $state_hr (@global_accept_state) {
-                            next unless $state_hr->{'src'} eq '0.0.0.0/0';
-                            next unless $state_hr->{'dst'} eq '0.0.0.0/0';
-                            next unless $state_hr->{'proto'} eq 'all' or
-                                $state_hr->{'proto'} = $rule{'proto'};
-                            next unless $state_hr->{'intf_in'} eq '*' or
-                                $state_hr->{'intf_in'} eq $rule{'intf_in'};
-                            next unless $state_hr->{'intf_out'} eq '*' or
-                                $state_hr->{'intf_out'} eq $rule{'intf_out'};
-                            ### if we make it here, then the state rule
-                            ### applies to this rule
-                            $rule{'state'} = $state_hr->{'state'};
-                        }
                     }
-                    if ($rule{'target'} eq 'ACCEPT'
-                            and $rule{'extended'} =~ m|^state\s+(\S+)|) {
-                        my $state_str = $1;
-                        if ($state_str =~ /ESTABLISHED/
-                                or $state_str =~ /RELATED/) {
-
-                            push @global_accept_state, {
-                                'state'    => $state_str,
-                                'src'      => $rule{'src'},
-                                'dst'      => $rule{'dst'},
-                                'intf_in'  => $rule{'intf_in'},
-                                'intf_out' => $rule{'intf_out'},
-                                'proto'    => $rule{'protocol'}
-                            };
-                            my %state_hash = ();
-                        }
+                    if ($rule{'extended'} =~ /\bctstate\s+(\S+)/) {
+                        $rule{'ctstate'} = $1;
+                    } elsif ($rule{'extended'} =~ /\bstate\s+(\S+)/) {
+                        $rule{'state'} = $1;
                     }
                 }
             }
         } else {
+
+            ### iptables:
             ### ACCEPT tcp  -- 164.109.8.0/24  0.0.0.0/0  tcp dpt:22 flags:0x16/0x02
             ### ACCEPT tcp  -- 216.109.125.67  0.0.0.0/0  tcp dpts:7000:7500
             ### ACCEPT udp  -- 0.0.0.0/0       0.0.0.0/0  udp dpts:7000:7500
@@ -246,9 +241,19 @@ sub chain_rules() {
 
             ### LOG  all  --  0.0.0.0/0  0.0.0.0/0  LOG flags 0 level 4 prefix `DROP '
             ### LOG  all  --  127.0.0.2  0.0.0.0/0  LOG flags 0 level 4
-            ### ### DNAT tcp  --  123.123.123.123  0.0.0.0/0  tcp dpt:55000 to:192.168.12.12:80
+            ### DNAT tcp  --  123.123.123.123  0.0.0.0/0  tcp dpt:55000 to:192.168.12.12:80
 
-            if ($line =~ m|^\s*(\S+)\s+(\S+)\s+\-\-\s+(\S+)\s+(\S+)\s*(.*)|) {
+            ### ip6tables:
+            ### ACCEPT     tcp   ::/0     fe80::aa:0:1/128    tcp dpt:12345
+            ### LOG        all   ::/0     ::/0                LOG flags 0 level 4
+
+            my $match_re = qr/^\s*(\S+)\s+(\S+)\s+\-\-\s+(\S+)\s+(\S+)\s*(.*)/;
+
+            if ($self->{'_ipt_bin_name'} eq 'ip6tables') {
+                $match_re = qr/^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*(.*)/;
+            }
+
+            if ($line =~ $match_re) {
                 $rule{'target'}   = $1;
                 my $proto = $2;
                 $proto = 'all' if $proto eq '0';
@@ -279,6 +284,12 @@ sub chain_rules() {
                         $rule{'to_ip'}   = $1;
                         $rule{'to_port'} = $2;
                     }
+
+                    if ($rule{'extended'} =~ /\bctstate\s+(\S+)/) {
+                        $rule{'ctstate'} = $1;
+                    } elsif ($rule{'extended'} =~ /\bstate\s+(\S+)/) {
+                        $rule{'state'} = $1;
+                    }
                 }
             }
         }
@@ -308,15 +319,16 @@ sub default_drop() {
         @ipt_lines = @$out_ar;
     }
 
-    return '[-] Could not get iptables output!', 0
+    return "[-] Could not get $self->{'_ipt_bin_name'} output!", 0
         unless @ipt_lines;
 
     my %protocols = ();
     my $found_chain = 0;
+    my $found_default_drop = 0;
     my $rule_ctr = 1;
     my $prefix;
     my $policy = 'ACCEPT';
-    my $any_ip_re = '(?:0\.){3}0/0';
+    my $any_ip_re = qr/(?:0\.){3}0\x2f0|\x3a{2}\x2f0/;
 
     LINE: for my $line (@ipt_lines) {
         chomp $line;
@@ -333,8 +345,20 @@ sub default_drop() {
         next LINE unless $found_chain;
 
         ### include ULOG target as well
-        if ($line =~ m|^\s*U?LOG\s+(\w+)\s+\-\-\s+.*
-            $any_ip_re\s+$any_ip_re\s+(.*)|x) {
+        my $log_re = qr/^\s*U?LOG\s+(\w+)\s+\-\-\s+.*
+                $any_ip_re\s+$any_ip_re\s+(.*)/x;
+        my $drop_re = qr/^DROP\s+(\w+)\s+\-\-\s+.*
+            $any_ip_re\s+$any_ip_re\s*$/x;
+
+        if ($self->{'_ipt_bin_name'} eq 'ip6tables') {
+            $log_re = qr/^\s*U?LOG\s+(\w+)\s+
+                    $any_ip_re\s+$any_ip_re\s+(.*)/x;
+            $drop_re = qr/^DROP\s+(\w+)\s+
+                $any_ip_re\s+$any_ip_re\s*$/x;
+        }
+
+        ### might as well pick up any default logging rules as well
+        if ($line =~ $log_re) {
             my $proto  = $1;
             my $p_tmp  = $2;
             my $prefix = 'NONE';
@@ -350,21 +374,28 @@ sub default_drop() {
             ### $proto may equal "all" here
             $protocols{$proto}{'LOG'}{'prefix'} = $prefix;
             $protocols{$proto}{'LOG'}{'rulenum'} = $rule_ctr;
-        } elsif ($policy eq 'ACCEPT' and $line =~ m|^DROP\s+(\w+)\s+\-\-\s+.*
-            $any_ip_re\s+$any_ip_re\s*$|x) {
+        } elsif ($policy eq 'ACCEPT' and $line =~ $drop_re) {
             my $proto = $1;
             $proto = 'all' if $proto eq '0';
             ### DROP    all  --  0.0.0.0/0     0.0.0.0/0
             $protocols{$1}{'DROP'} = $rule_ctr;
+            $found_default_drop = 1;
         }
         $rule_ctr++;
     }
+
     ### if the policy in the chain is DROP, then we don't
     ### necessarily need to find a default DROP rule.
     if ($policy eq 'DROP') {
         $protocols{'all'}{'DROP'} = 0;
+        $found_default_drop = 1;
     }
-    return \%protocols;
+
+    return "[-] There are no default drop rules in the " .
+            "$self->{'_ipt_bin_name'} policy!", 0
+        unless %protocols and $found_default_drop;
+
+    return \%protocols, 1;
 }
 
 sub default_log() {
@@ -374,7 +405,7 @@ sub default_log() {
     my $file  = shift || '';
     my $iptables  = $self->{'_iptables'};
 
-    my $any_ip_re  = '(?:0\.){3}0/0';
+    my $any_ip_re  = qr/(?:0\.){3}0\x2f0|\x3a{2}\x2f0/;
     my @ipt_lines  = ();
     my %log_chains = ();
     my %log_rules  = ();
@@ -405,7 +436,7 @@ sub default_log() {
         }
     }
 
-    return '[-] Could not get iptables output!', 0
+    return "[-] Could not get $self->{'_ipt_bin_name'} output!", 0
         unless @ipt_lines;
 
     ### first get all logging rules and associated chains
@@ -426,17 +457,34 @@ sub default_log() {
         my $proto = '';
         my $found = 0;
         if ($ipt_verbose) {
-            if ($line =~ m|^\s*\d+\s+\d+\s*U?LOG\s+(\w+)\s+\-\-\s+
-                    \S+\s+\S+\s+$any_ip_re
-                    \s+$any_ip_re\s+.*U?LOG|x) {
-                $proto = $1;
-                $found = 1;
+            if ($self->{'_ipt_bin_name'} eq 'ip6tables') {
+                if ($line =~ m|^\s*\d+\s+\d+\s*U?LOG\s+(\w+)\s+
+                        \S+\s+\S+\s+$any_ip_re
+                        \s+$any_ip_re\s+.*U?LOG|x) {
+                    $proto = $1;
+                    $found = 1;
+                }
+            } else {
+                if ($line =~ m|^\s*\d+\s+\d+\s*U?LOG\s+(\w+)\s+\-\-\s+
+                        \S+\s+\S+\s+$any_ip_re
+                        \s+$any_ip_re\s+.*U?LOG|x) {
+                    $proto = $1;
+                    $found = 1;
+                }
             }
         } else {
-            if ($line =~ m|^\s*U?LOG\s+(\w+)\s+\-\-\s+$any_ip_re
-                    \s+$any_ip_re\s+.*U?LOG|x) {
-                $proto = $1;
-                $found = 1;
+            if ($self->{'_ipt_bin_name'} eq 'ip6tables') {
+                if ($line =~ m|^\s*U?LOG\s+(\w+)\s+$any_ip_re
+                        \s+$any_ip_re\s+.*U?LOG|x) {
+                    $proto = $1;
+                    $found = 1;
+                }
+            } else {
+                if ($line =~ m|^\s*U?LOG\s+(\w+)\s+\-\-\s+$any_ip_re
+                        \s+$any_ip_re\s+.*U?LOG|x) {
+                    $proto = $1;
+                    $found = 1;
+                }
             }
         }
 
@@ -448,8 +496,8 @@ sub default_log() {
         }
     }
 
-    return '[-] There are no logging rules in the iptables policy!', 0
-        unless %log_chains;
+    return "[-] There are no default logging rules " .
+        "in the $self->{'_ipt_bin_name'} policy!", 0 unless %log_chains;
 
     my %sub_chains = ();
 
@@ -467,7 +515,7 @@ sub default_log() {
         }
     }
 
-    return \%log_rules;
+    return \%log_rules, 1;
 }
 
 sub sub_chains() {
@@ -486,7 +534,7 @@ sub sub_chains() {
         if ($found and $line =~ /^\s*Chain\s/) {
             last;
         }
-        if ($line =~ m|^\s*(\S+)\s+\S+\s+\-\-|) {
+        if ($line =~ m|^\s*(\S+)\s+\S+\s+|) {
             my $new_chain = $1;
             if ($new_chain ne 'LOG'
                     and $new_chain ne 'DROP'
@@ -496,7 +544,10 @@ sub sub_chains() {
                     and $new_chain ne 'QUEUE'
                     and $new_chain ne 'SNAT'
                     and $new_chain ne 'DNAT'
-                    and $new_chain ne 'MASQUERADE') {
+                    and $new_chain ne 'MASQUERADE'
+                    and $new_chain ne 'pkts'
+                    and $new_chain ne 'Chain'
+                    and $new_chain ne 'target') {
                 $chains_href->{$new_chain} = '';
                 &sub_chains($new_chain, $chains_href, $ipt_lines_aref);
             }
@@ -507,7 +558,8 @@ sub sub_chains() {
 
 sub exec_iptables() {
     my $self  = shift;
-    my $cmd = shift || croak '[*] Must specify an iptables command to run.';
+    my $cmd = shift || croak "[*] Must specify an " .
+        "$self->{'_ipt_bin_name'} command to run.";
     my $iptables  = $self->{'_iptables'};
     my $iptout    = $self->{'_iptout'};
     my $ipterr    = $self->{'_ipterr'};
@@ -518,8 +570,9 @@ sub exec_iptables() {
     my $ipt_exec_sleep = $self->{'_ipt_exec_sleep'};
     my $sigchld_handler = $self->{'_sigchld_handler'};
 
-    croak "[*] $cmd does not look like an iptables command."
-        unless $cmd =~ m|^\s*iptables| or $cmd =~ m|^\S+/iptables|;
+    croak "[*] $cmd does not look like an $self->{'_ipt_bin_name'} command."
+        unless $cmd =~ m|^\s*iptables| or $cmd =~ m|^\S+/iptables|
+            or $cmd =~ m|^\s*ip6tables| or $cmd =~ m|^\S+/ip6tables|;
 
     my $rv = 1;
     my @stdout = ();
@@ -538,10 +591,10 @@ sub exec_iptables() {
     }
 
     if ($ipt_exec_sleep > 0) {
-    	if ($debug or $verbose) {
+        if ($debug or $verbose) {
             print $fh localtime() . " [+] IPTables::Parse: ",
                 "sleeping for $ipt_exec_sleep seconds before ",
-                "executing iptables command.\n";
+                "executing $self->{'_ipt_bin_name'} command.\n";
         }
         sleep $ipt_exec_sleep;
     }
@@ -558,7 +611,7 @@ sub exec_iptables() {
     } else {
         my $ipt_pid;
 
-    	if ($debug or $verbose) {
+        if ($debug or $verbose) {
             print $fh localtime() . " [+] IPTables::Parse: " .
                 "Setting SIGCHLD handler to: " . $sigchld_handler . "\n";
         }
@@ -569,7 +622,8 @@ sub exec_iptables() {
                 ### iptables should never take longer than 30 seconds to execute,
                 ### unless there is some absolutely enormous policy or the kernel
                 ### is exceedingly busy
-                local $SIG{'ALRM'} = sub {die "[*] iptables command timeout.\n"};
+                local $SIG{'ALRM'} = sub {die "[*] $self->{'_ipt_bin_name'} " .
+                    "command timeout.\n"};
                 alarm $ipt_alarm;
                 waitpid($ipt_pid, 0);
                 alarm 0;
@@ -578,7 +632,7 @@ sub exec_iptables() {
                 kill 9, $ipt_pid unless kill 15, $ipt_pid;
             }
         } else {
-            croak "[*] Could not fork iptables: $!"
+            croak "[*] Could not fork $self->{'_ipt_bin_name'}: $!"
                 unless defined $ipt_pid;
 
             ### exec the iptables command and preserve stdout and stderr
@@ -600,7 +654,8 @@ sub exec_iptables() {
     }
 
     if ($debug or $verbose) {
-        print $fh localtime() . "     iptables command stdout:\n";
+        print $fh localtime() . "     $self->{'_ipt_bin_name'} " .
+            "command stdout:\n";
         for my $line (@stdout) {
             if ($line =~ /\n$/) {
                 print $fh $line;
@@ -608,7 +663,8 @@ sub exec_iptables() {
                 print $fh $line, "\n";
             }
         }
-        print $fh localtime() . "     iptables command stderr:\n";
+        print $fh localtime() . "     $self->{'_ipt_bin_name'} " .
+            "command stderr:\n";
         for my $line (@stderr) {
             if ($line =~ /\n$/) {
                 print $fh $line;
@@ -635,14 +691,16 @@ __END__
 
 =head1 NAME
 
-IPTables::Parse - Perl extension for parsing iptables firewall rulesets
+IPTables::Parse - Perl extension for parsing iptables and ip6tables policies
 
 =head1 SYNOPSIS
 
   use IPTables::Parse;
 
+  my $ipt_bin = '/sbin/iptables'; # can set this to /sbin/ip6tables
+
   my %opts = (
-      'iptables' => '/sbin/iptables',
+      'iptables' => $ipt_bin,
       'iptout'   => '/tmp/iptables.out',
       'ipterr'   => '/tmp/iptables.err',
       'debug'    => 0,
@@ -662,14 +720,14 @@ IPTables::Parse - Perl extension for parsing iptables firewall rulesets
       if (defined $ipt_hr->{'all'}) {
           print "The INPUT chain has a default DROP rule for all protocols.\n";
       } else {
-          for my $proto qw/tcp udp icmp/ {
+          for my $proto (qw/tcp udp icmp/) {
               if (defined $ipt_hr->{$proto}) {
                   print "The INPUT chain drops $proto by default.\n";
               }
           }
       }
   } else {
-      print "[-] Could not parse iptables policy\n";
+      print "[-] Could not parse $ipt_obj->{'_ipt_bin_name'} policy\n";
   }
 
   ($ipt_hr, $rv) = $ipt_obj->default_log($table, $chain);
@@ -677,23 +735,24 @@ IPTables::Parse - Perl extension for parsing iptables firewall rulesets
       if (defined $ipt_hr->{'all'}) {
           print "The INPUT chain has a default LOG rule for all protocols.\n";
       } else {
-          for my $proto qw/tcp udp icmp/ {
+          for my $proto (qw/tcp udp icmp/) {
               if (defined $ipt_hr->{$proto}) {
                   print "The INPUT chain logs $proto by default.\n";
               }
           }
       }
   } else {
-      print "[-] Could not parse iptables policy\n";
+      print "[-] Could not parse $ipt_obj->{'_ipt_bin_name'} policy\n";
   }
 
 =head1 DESCRIPTION
 
-The C<IPTables::Parse> package provides an interface to parse iptables
-rules on Linux systems through the direct execution of iptables commands, or
-from parsing a file that contains an iptables policy listing.  You can get the
-current policy applied to a table/chain, look for a specific user-defined chain,
-check for a default DROP policy, or determing whether or not logging rules exist.
+The C<IPTables::Parse> package provides an interface to parse iptables or
+ip6tables rules on Linux systems through the direct execution of
+iptables/ip6tables commands, or from parsing a file that contains an
+iptables/ip6tables policy listing.  You can get the current policy applied to a
+table/chain, look for a specific user-defined chain, check for a default DROP
+policy, or determing whether or not logging rules exist.
 
 =head1 FUNCTIONS
 
@@ -717,27 +776,27 @@ the following keys (that contain values depending on the rule): C<src>, C<dst>,
 C<protocol>, C<s_port>, C<d_port>, C<target>, C<packets>, C<bytes>, C<intf_in>,
 C<intf_out>, C<to_ip>, C<to_port>, C<state>, C<raw>, and C<extended>.  The C<extended>
 element contains the rule output past the protocol information, and the C<raw>
-element contains the complete rule itself as reported by iptables.
+element contains the complete rule itself as reported by iptables or ip6tables.
 
 =item default_drop($table, $chain)
 
-This function parses the running iptables policy in order to determine if
-the specified chain contains a default DROP rule.  Two values are returned,
-a hash reference whose keys are the protocols that are dropped by default
-if a global ACCEPT rule has not accepted matching packets first, along with
-a return value that tells the caller if parsing the iptables policy was
-successful.  Note that if all protocols are dropped by default, then the
-hash key 'all' will be defined.
+This function parses the running iptables or ip6tables policy in order to
+determine if the specified chain contains a default DROP rule.  Two values
+are returned, a hash reference whose keys are the protocols that are dropped by
+default if a global ACCEPT rule has not accepted matching packets first, along
+with a return value that tells the caller if parsing the iptables or ip6tables
+policy was successful.  Note that if all protocols are dropped by default, then
+the hash key 'all' will be defined.
 
   ($ipt_hr, $rv) = $ipt_obj->default_drop('filter', 'INPUT');
 
 =item default_log($table, $chain)
 
-This function parses the running iptables policy in order to determine if
+This function parses the running iptables or ip6tables policy in order to determine if
 the specified chain contains a default LOG rule.  Two values are returned,
 a hash reference whose keys are the protocols that are logged by default
 if a global ACCEPT rule has not accepted matching packets first, along with
-a return value that tells the caller if parsing the iptables policy was
+a return value that tells the caller if parsing the iptables or ip6tables policy was
 successful.  Note that if all protocols are logged by default, then the
 hash key 'all' will be defined.  An example invocation is:
 
@@ -752,21 +811,27 @@ Michael Rash, E<lt>mbr@cipherdyne.orgE<gt>
 =head1 SEE ALSO
 
 The IPTables::Parse is used by the IPTables::ChainMgr extension in support of
-the psad, fwsnort, and fwknop projects to parse iptables policies (see the psad(8),
-fwsnort(8), and fwknop(8) man pages).  As always, the iptables(8) provides the
-best information on command line execution and theory behind iptables.
+the psad and fwsnort projects to parse iptables or ip6tables policies (see the psad(8),
+and fwsnort(8) man pages).  As always, the iptables(8) and ip6tables(8) man pages
+provide the best information on command line execution and theory behind iptables
+and ip6tables.
 
 Although there is no mailing that is devoted specifically to the IPTables::Parse
 extension, questions about the extension will be answered on the following
 lists:
 
   The psad mailing list: http://lists.sourceforge.net/lists/listinfo/psad-discuss
-  The fwknop mailing list: http://lists.sourceforge.net/lists/listinfo/fwknop-discuss
   The fwsnort mailing list: http://lists.sourceforge.net/lists/listinfo/fwsnort-discuss
 
-The latest version of the IPTables::Parse extension can be found at:
+The latest version of the IPTables::Parse extension can be found on CPAN and
+also here:
 
-http://www.cipherdyne.org/modules/
+  http://www.cipherdyne.org/modules/
+
+Source control is provided by git:
+
+  http://www.cipherdyne.org/git/IPTables-Parse.git
+  http://www.cipherdyne.org/cgi-bin/gitweb.cgi?p=IPTables-Parse.git;a=summary
 
 =head1 CREDITS
 
@@ -778,15 +843,19 @@ Thanks to the following people:
 =head1 AUTHOR
 
 The IPTables::Parse extension was written by Michael Rash F<E<lt>mbr@cipherdyne.orgE<gt>>
-to support the psad, fwknop, and fwsnort projects.  Please send email to
+to support the psad and fwsnort projects.  Please send email to
 this address if there are any questions, comments, or bug reports.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-2008 by Michael Rash
+Copyright (C) 2005-2012 Michael Rash.  All rights reserved.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.5 or,
-at your option, any later version of Perl 5 you may have available.
+This module is free software.  You can redistribute it and/or
+modify it under the terms of the Artistic License 2.0.  More information
+can be found here: http://www.perl.com/perl/misc/Artistic.html
+
+This program is distributed "as is" in the hope that it will be useful,
+but without any warranty; without even the implied warranty of
+merchantability or fitness for a particular purpose.
 
 =cut
